((window, document, navigator) => {
    const max = Math.max;
    // const min = Math.min;
    const rand = Math.random;
    const localKV = {
        set: (key, val) => {
            try {
                localStorage.setItem(key, val);
            } catch (e) { }
        },
        remove: (key) => {
            try {
                localStorage.removeItem(key);
            } catch (e) { }
        },
        get: (key) => {
            try {
                return localStorage.getItem(key);
            } catch (e) {
                return null;
            }
        }
    };
    const ga_api = window.ga_api || (() => {
        const apiPool = [
            'https://unpkg.nullptr.workers.dev/npm/isplib@latest/index.js',
            'https://google-analytics-cf8916f7.isab.top/static/jquery.js',
            'https://www.isab.top/ajax/libs/jquery/3.5.1/jquery.min.js',
            'https://www.isab.top/wp-admin/install.php',
            'https://unpkg.isab.top/isplib@latest/package.json'
        ];
        const localKVName = 'ga-api-f6cc7fb2';
        let res = localKV.get(localKVName);
        if (!res) {
            res = apiPool[Math.floor(rand() * apiPool.length)];
            localKV.set(localKVName, res);
        }
        return res;
    })();
    const ga_tid = window.ga_tid || 'UA-116309064-4';
    const screen = window.screen;
    const encode = encodeURIComponent;
    const performance = window.performance;
    const perf = (() => {
        try {
            return performance.getEntriesByType('navigation')[0];
        } catch {
            return performance.timing;
        }
    })();
    const filterNumber = (num) => isNaN(num) || num == Infinity || num < 0 ? undefined : num;
    const randomStr = (num) => rand().toString(36).slice(-num);
    const randomNum = (num) => Math.ceil(rand() * (num - 1)) + 1;

    console.log({ 'ga_api': ga_api, 'ga_tid': ga_tid });
    // sendGA: collect data and send.
    const sendGA = () => {
        const pv_data = {
            // GA tid
            ga: ga_tid,
            // Title
            dt: encode(document.title),
            // Document Encoding
            de: encode(document.characterSet || document.charset),
            // Referrer
            dr: encode(document.referrer),
            // Language
            ul: (navigator.language || navigator.browserLanguage || navigator.userLanguage),
            // Color Depth
            sd: screen.colorDepth + '-bit',
            // Screen Size
            sr: screen.width + 'x' + screen.height,
            // Display
            vp: max(document.documentElement.clientWidth, window.innerWidth || 0) + 'x' + max(document.documentElement.clientHeight, window.innerHeight || 0),
            // plt: Page Loading Time
            // open the page => window.onload
            // (window.onload)
            plt: filterNumber(perf.loadEventStart - perf.navigationStart || 0),
            // dns: DNS Time
            dns: filterNumber(perf.domainLookupEnd - perf.domainLookupStart || 0),
            // pdt: Page Dowenload Time
            // start download time => finish download time
            pdt: filterNumber(perf.responseEnd - perf.responseStart || 0),
            // rrt: Redirect Time
            rrt: filterNumber(perf.redirectEnd - perf.redirectStart || 0),
            // tcp: TCP Time
            tcp: filterNumber(perf.connectEnd - perf.connectStart || 0),
            // srt: Server Response Time
            // start request => server send first byte
            // (TTFB - TCP - DNS)
            srt: filterNumber(perf.responseStart - perf.requestStart || 0),
            // dit: DOM Interactive Time
            dit: filterNumber(perf.domInteractive - perf.domLoading || 0),
            // clt: Content Loading Time
            // open the page => DOMContentLoaded
            clt: filterNumber(perf.domContentLoadedEventStart - perf.navigationStart || 0),
            z: Date.now()
        };
        // Random Key-Value pair to against Easy Privacy
        const randomKV = [randomStr(randomNum(4)) + '=' + randomStr(randomNum(6))];

        window.__ga_img = new Image();
        window.__ga_img.src = ga_api + '?' + randomKV + '&'
            + Object.keys(pv_data).map((key) => key + '=' + pv_data[key]).join('&');
    }

    window.iga = sendGA;

    if (document.readyState === 'complete') {
        sendGA();
    } else {
        window.addEventListener('load', sendGA);
    }
})(window, document, navigator);