export const getStats = ((document) => {
    return async (pkgName) => {
        const divName = `${pkgName}-${Date.now()}`;
        const dataDiv = document.createElement('div');
        dataDiv.id = divName;

        const title = document.createElement('h3');
        title.id = `${divName}-title`;
        title.innerText = `Package ${pkgName}`;
        dataDiv.appendChild(title);

        const badgeImg = document.createElement('img');
        badgeImg.id = `${divName}-badge`;
        dataDiv.appendChild(badgeImg);

        const statsDiv = document.createElement('div');
        statsDiv.id = `${divName}-stats`;
        dataDiv.appendChild(statsDiv);

        const errorDiv = document.createElement('div');
        errorDiv.id = `${divName}-error`;
        errorDiv.style = 'color: red;';
        dataDiv.appendChild(errorDiv);

        const createTextDiv = (text) => {
            const div = document.createElement('div');
            div.id = `text-${Date.now()}`;
            div.innerText = text;
            return div;
        };
        await Promise.all([
            fetch(`https://data.jsdelivr.com/v1/package/npm/${pkgName}/badge`, { cache: 'no-store' })
                .then(res => res.ok ? res.blob() : res.text().then(text => Promise.reject(`Failed to load badge. Error: \n${text}`)))
                .then(blob => badgeImg.src = URL.createObjectURL(blob))
                .catch((err) => {
                    badgeImg.hidden = true;
                    errorDiv.appendChild(createTextDiv(err));
                }),
            fetch(`https://data.jsdelivr.com/v1/package/npm/${pkgName}/stats`, { cache: 'no-store' })
                .then(res => res.ok ? res.json() : res.text().then(text => Promise.reject(`Failed to load stats. Error: \n${text}`)))
                .then(json => statsDiv.innerText = `rank: ${json.rank}\ntotal: ${json.total}\n`)
                .catch((err) => {
                    statsDiv.hidden = true;
                    errorDiv.appendChild(createTextDiv(err));
                })
        ]);

        return dataDiv;
    };
})(document);