(async (window, document, console) => {
    const fillURL = (fileName) => `./loadImage/${fileName}`;
    const preloadList = ['lib.js', 'listURL.js'];

    await Promise.all(
        [
            // fetch style
            fetch(new URL(fillURL('style.css'), document.currentScript.src).href, { cache: 'force-cache' })
                .then(res => res.text())
                .then((styleText) => {
                    const style = document.createElement('style');
                    style.innerHTML = styleText;
                    document.head.appendChild(style);
                })
                .catch(err => console.warn(`Style Error: ${err}`)),
            // preload files
            Promise.all(preloadList.map(resource => import(fillURL(resource))
                .catch(err => console.warn(`Preload "${resource}" Error: ${err}`)))),
            // import subscript
            import(fillURL(window.caches ? 'withCache.js' : 'withoutCache.js'))
                .then((subscript) => {
                    const slide = '_slide' in window;
                    const slideDur = 60 * 1000;  // time duration between two images
                    const blurType = window._blurType || 'back';
                    const imgPlace = ((elem) => {
                        if (typeof elem === 'string') {
                            return document.getElementById(elem);
                        }
                        return elem;
                    })(window._place);
                    const showOn = (place) => {
                        if (blurType === 'no') {
                            place.classList.add('no-blur');
                        } else if (blurType === 'back') {
                            place.classList.add('blur');
                        } else if (blurType === 'self') {
                            place.classList.add('blur-self');
                        }
                        const tagName = place.tagName.toLowerCase();
                        const nextImage = () => {
                            subscript.getImgBlobURL()
                                .then((blobURL) => {
                                    if (blobURL) {
                                        if (tagName === 'img') {
                                            place.src = blobURL;
                                        } else {
                                            place.style.backgroundImage = `url(${blobURL})`;
                                        }
                                    }
                                });
                        };
                        if (slide) {
                            window.addEventListener('online', nextImage);
                            nextImage();
                            setInterval(nextImage, slideDur);
                        } else {
                            nextImage();
                        }
                    };
                    if (imgPlace) {
                        showOn(imgPlace);
                    } else {
                        if (document.readyState === 'complete') {
                            showOn(document.body);
                        } else {
                            document.addEventListener('DOMContentLoaded', () => {
                                showOn(document.body);
                            });
                        }
                    }
                })
                .catch(err => console.warn(`Subscript Error: ${err}`))
        ]
    );
})(window, document, console);