import {
    listName, imgName, listURL,
    debugNetwork, searchURL, clearDiv
} from './lib.js';
import {
    abortOnHidden
} from './abortOnHidden.js';

export default ((window, document) => {
    return async (showDiv, button) => {
        abortOnHidden(showDiv, button);

        const divTitle = '已缓存的 URL';
        showDiv.appendChild(document.createTextNode(divTitle));

        const makeDiv = (divId, text) => {
            const div = document.createElement('div');
            div.id = divId;
            if (text) {
                const textNode = document.createTextNode(text);
                div.appendChild(textNode);
            } else {
                div.hidden = true;
            }
            return div;
        };

        let previousDiv = makeDiv(Date.now());
        showDiv.appendChild(previousDiv);
        let childCount = 0;

        let pendingClear = false;

        const appendData = (text) => {
            const latestDiv = makeDiv(Date.now(), text);
            if (childCount >= 200) {
                if (!pendingClear) {
                    pendingClear = true;
                    requestAnimationFrame(() => {
                        clearDiv(showDiv);
                        showDiv.appendChild(document.createTextNode(divTitle));
                        childCount = 0;
                        pendingClear = false;
                    });
                }
                previousDiv = makeDiv(Date.now());
                showDiv.appendChild(previousDiv);
            }
            showDiv.insertBefore(latestDiv, previousDiv);
            ++childCount;
            previousDiv = latestDiv;
        };

        const listCache = window.caches ? await caches.open(listName) : undefined;
        const imgCache = window.caches ? await caches.open(imgName) : undefined;

        const procList = async (url) => {
            const procResource = async (url) => {
                const request = new Request(url);
                await fetch(request, { cache: 'force-cache' })
                    .then(async (response) => {
                        if (imgCache) {
                            await imgCache.put(request, response);
                        }
                    })
                    .then(() => appendData(url))
                    .catch(err => debugNetwork(false, url, err));
            };

            const request = new Request(url);
            const listJSON = await fetch(request, { cache: 'force-cache' })
                .then(async (response) => {
                    if (listCache) {
                        await listCache.put(request, response.clone());
                    }
                    return await response.json();
                })
                .catch(err => debugNetwork(false, url, err));
            if (listJSON) {
                const resourceURL = searchURL(listJSON);
                await Promise.all(resourceURL.map(procResource));
            }
        };

        await Promise.all(listURL.map(procList));
    };
})(window, document);