// Functions
const choice = list => list[Math.floor(Math.random() * list.length)];
const key = (obj, fallback = undefined) => { const keys = Object.keys(obj); return keys.length ? choice(keys) : fallback; };
const value = (obj, fallback = undefined) => { const keys = Object.keys(obj); return keys.length ? obj[choice(keys)] : fallback; };
const createBlobURL = (blob, url, cached) => {
    // caller should ensure that url is not in cache
    const blobURL = URL.createObjectURL(blob);
    cached[url] = blobURL;
    return blobURL;
};
const searchURL = (obj) => {
    const res = [];
    const recursion = (obj) => {
        if (typeof obj === 'object') {
            const values = Object.values(obj);
            for (const val of values) {
                if (typeof val === 'string') {
                    if (val.startsWith('https://')) {
                        res.push(val);
                    }
                } else if (typeof val === 'object') {
                    recursion(val);
                } else {
                    console.warn(`What's this? \n${val}`);
                }
            }
        }
    };
    recursion(obj);
    return res;
};
const delayedCall = (func, document) => {
    if (document.readyState === 'complete') {
        func();
    } else {
        document.addEventListener('DOMContentLoaded', func);
    }
};
const clearDiv = (div) => {
    let cur = div.firstChild;
    while (cur) {
        div.removeChild(cur);
        cur = div.firstChild;
    }
};

// Values
const listName = 'list-8ea85253';
const imgName = 'image-b26452a3';
import {
    listURL
} from './listURL.js';
const isWebpSupported = (document.createElement('canvas').toDataURL('image/webp').indexOf('data:image/webp') === 0);

// Debuggers
const debug = (eventType, eventStatus, eventDetail, isSucceeded) => {
    if (isSucceeded === false) {
        console.warn(`${eventType}: ${eventStatus}\nDetail:`, eventDetail);
    } else {
        console.log(`${eventType}: ${eventStatus}\nDetail:`, eventDetail);
    }
}
const debugNetwork = (isSucceeded, resourceURL, errorReason) => {
    const eventType = 'Network';
    const eventStatus = isSucceeded ? 'Succeeded' : 'Failed';
    const eventDetail = {
        url: resourceURL,
        reason: errorReason
    };
    debug(eventType, eventStatus, eventDetail, isSucceeded);
};
const debugCache = (isHit, resourceURL) => {
    const eventType = 'Cache';
    const eventStatus = isHit ? 'Hit' : 'Missed';
    const eventDetail = {
        url: resourceURL
    };
    debug(eventType, eventStatus, eventDetail, isHit);
};

export {
    choice, key, value, createBlobURL, searchURL, delayedCall, clearDiv,  // Functions
    listName, imgName, listURL, isWebpSupported, // Values
    debug, debugNetwork, debugCache, // Debuggers
};