import {
    listURL,
    debugNetwork, searchURL, clearDiv
} from './lib.js';
import {
    abortOnHidden
} from './abortOnHidden.js';

export default ((document) => {
    return async (showDiv, button) => {
        abortOnHidden(showDiv, button);

        const divTitle = '已加载的 URL';
        showDiv.appendChild(document.createTextNode(divTitle));

        const makeDiv = (divId, text) => {
            const div = document.createElement('div');
            div.id = divId;
            if (text) {
                const textNode = document.createTextNode(text);
                div.appendChild(textNode);
            } else {
                div.hidden = true;
            }
            return div;
        };

        let previousDiv = makeDiv(Date.now());
        showDiv.appendChild(previousDiv);
        let childCount = 0;

        let pendingClear = false;

        const appendData = (text) => {
            const latestDiv = makeDiv(Date.now(), text);
            if (childCount >= 200) {
                if (!pendingClear) {
                    pendingClear = true;
                    requestAnimationFrame(() => {
                        clearDiv(showDiv);
                        showDiv.appendChild(document.createTextNode(divTitle));
                        childCount = 0;
                        pendingClear = false;
                    });
                }
                previousDiv = makeDiv(Date.now());
                showDiv.appendChild(previousDiv);
            }
            showDiv.insertBefore(latestDiv, previousDiv);
            ++childCount;
            previousDiv = latestDiv;
        };

        await Promise.all(
            listURL.map(url => fetch(url, { cache: 'no-store' })
                .then(res => res.json())
                .then(json => Promise.all(searchURL(json).map(url => fetch(url, { cache: 'no-store' })
                    .then(() => appendData(url))
                    .catch(err => debugNetwork(false, url, err)))
                ))
                .catch(err => debugNetwork(false, url, err))));
    };
})(document);