import {
    delayedCall, debugNetwork, clearDiv
} from './lib.js';

(async (document) => {
    const showDiv = document.createElement('div');
    const widgetMap = {
        cacheToLocal: {
            name: '缓存图片到本地',
            path: './cacheToLocal.js',
        },
        listResource: {
            name: '列出图片',
            path: './listResource.js'
        },
        pinToCDN: {
            name: '要求 CDN 缓存图片',
            path: './pinToCDN.js'
        },
        search: {
            name: '查找图片',
            path: './search.js'
        }
    };
    const widgetList = Object.keys(widgetMap);
    await Promise.all(widgetList.map(async (key) => {
        const subscript = await import(widgetMap[key].path)
            .catch((err) => debugNetwork(false, widgetMap[key], err));
        if (!subscript) {
            return;
        }
        widgetMap[key].func = subscript.default;
    }));

    widgetList.map((key) => {
        const button = document.createElement('button');
        const div = document.createElement('div');
        const name = widgetMap[key].name;
        const func = widgetMap[key].func;
        let display = true;
        const nextStatus = async () => {
            if (display === false) {
                display = true;
                div.hidden = false;
                button.innerText = name + ' 🔄';
                try {
                    await func(div, button);
                } catch { /* 子模块通过 throw new Error 来退出 */ }
                button.innerText = name + ' 🔼';
            } else {
                display = false;
                div.hidden = true;
                button.innerText = name + ' 🔽';
                clearDiv(div);
            }
        };

        button.onclick = nextStatus;
        nextStatus();

        showDiv.appendChild(button);
        showDiv.appendChild(div);
    });

    delayedCall(() => {
        document.title = '工具箱';
        document.body.appendChild(showDiv);
    }, document);
})(document);