import {
    choice, createBlobURL,
    listName, imgName, isWebpSupported, listURL,
    debugNetwork, debugCache
} from './lib.js';

const createdBlob = {};

export const getImgBlobURL = async () => {
    const cacheBucket = await Promise.all([
        caches.open(listName),
        caches.open(imgName)
    ]);
    const listCache = cacheBucket[0], imgCache = cacheBucket[1];
    const getImgList = async (url) => {
        const request = new Request(url);
        const cachedResponse = await listCache.match(request)
            .then(res => res.json())
            .catch(() => debugCache(false, url));
        if (cachedResponse) {
            debugCache(true, url);
            return cachedResponse;
        } else {
            const response = await fetch(request)
                .catch(err => debugNetwork(false, url, err));
            if (response) {
                debugNetwork(true, url);
                await listCache.put(request, response);
                return await listCache.match(request)
                    .then(res => res.json());
            } else {
                const imgListCacheKeys = await listCache.keys()
                if (imgListCacheKeys.length) {
                    return await listCache.match(choice(imgListCacheKeys))
                        .then(res => res.json());
                } else {
                    return undefined;
                }
            }
        }
    };
    const getImg = async (url) => {
        if (url in createdBlob) {
            debugCache(true, url);
            return createdBlob[url];
        } else {
            const request = new Request(url);
            const cachedResponse = await imgCache.match(request)
                .then(res => res.blob())
                .then(blob => createBlobURL(blob, url, createdBlob))
                .catch(() => debugCache(false, url));
            if (cachedResponse) {
                debugCache(true, url);
                return cachedResponse;
            } else {
                const response = await fetch(request)
                    .catch(err => debugNetwork(false, url, err));
                if (response) {
                    debugNetwork(true, url);
                    await imgCache.put(request, response);
                    const respBlob = await imgCache.match(request)
                        .then(res => res.blob());
                    return createBlobURL(respBlob, url, createdBlob);
                } else {
                    const imgCacheKeys = await imgCache.keys();
                    if (imgCacheKeys.length) {
                        const resp = await imgCache.match(choice(imgCacheKeys));
                        const respURL = resp.url;
                        if (respURL in createdBlob) {
                            return createdBlob[respURL];
                        } else {
                            const respBlob = await resp.blob();
                            return createBlobURL(respBlob, respURL, createdBlob);
                        }
                    } else {
                        return undefined;
                    }
                }
            }
        }
    };
    const imgList = await getImgList(choice(listURL));
    if (!imgList) {
        return undefined;
    } else {
        const img = choice(imgList);
        const imgURL = isWebpSupported && img.webp ? img.webp : img.fallback;
        if (!imgURL) {
            return undefined;
        } else {
            return await getImg(imgURL);
        }
    }
};